@@ -39,7 +39,7 @@ pub fn MovieOverview(movie: &Movie) {
         Image(cx, &movie.img_url);
         RandomRenderObject(cx, &movie.name);
 
-        let count = cx.state(Rc::new(RefCell::new(0usize)));
+        let count = cx.remember(Rc::new(RefCell::new(0usize)));
         Text(cx, format!("compose count {}", count.borrow()));
         *count.borrow_mut() += 1;
     });
@@ -58,7 +58,7 @@ fn main() {
 
     // first run
     let movies = vec![Movie::new(1, "A", "IMG_A"), Movie::new(2, "B", "IMG_B")];
-    root_fn(recomposer.composer(), &movies);
+    root_fn(recomposer.cx(), &movies);
 
     // end compose
     recomposer.finalize();
@@ -74,7 +74,7 @@ fn main() {
         Movie::new(3, "C", "IMG_C"),
         Movie::new(2, "B", "IMG_B"),
     ];
-    root_fn(recomposer.composer(), &movies);
+    root_fn(recomposer.cx(), &movies);
 
     recomposer.finalize();
     // end compose, Recomposer allow you to access root
